@@ -1,4 +1,5 @@
 import argparse
+import importlib
 import os
 import shutil
 import sys
@@ -57,21 +58,22 @@ if __name__ == "__main__":
     # will expect it to be sane.
     sys.argv[:] = args.script
 
-    # Read in the script to be executed and compile their code.
-    # NOTE: This looks pretty questionable, but it's effectively equivalent to
-    # what cProfile is doing, so it can't be that bad.
+    # Read in the script to be executed and compile their code, then ensure it
+    # appears to be a normal module.
+    #
+    # NOTE: This is pretty sketchy and requires some care to make work properly
+    # in all cases.  In particular, use of `__file__` and
+    # `multiprocessing.Pool` are good sanity checks if this is modified.
     sys.path.insert(0, os.path.dirname(sys.argv[0]))
     with open(sys.argv[0], 'rb') as fp:
         code = compile(fp.read(), sys.argv[0], 'exec')
 
-    # exec() will maintain our existing globals unless we override them, which
-    # will leave __file__ in particular set to the wrong value.
-    new_globals = {
-        '__file__': sys.argv[0],
-        '__name__': '__main__',
-        '__package__': None,
-        '__cached__': None,
-    }
+    # Load the file as __main__ and insert it into the set of modules.  This
+    # won't do anything useful on its own, but is necessary for the exec() call
+    # below, which needs to operate in the context of our new module.
+    spec = importlib.util.spec_from_file_location('__main__', sys.argv[0])
+    mod = importlib.util.module_from_spec(spec)
+    sys.modules.update({ "__main__": mod})
 
     # Ensure we're setup to be able to run.
     setup_dependencies()
@@ -81,8 +83,9 @@ if __name__ == "__main__":
         _functiontrace.config_tracememory()
     _functiontrace.begin_tracing(args.output_dir)
 
-    # Run their code now that we're tracing.
-    exec(code, new_globals)
+    # Run their code now that we're tracing.  This must be done in the context
+    # of the __main__ module we've created.
+    exec(code, mod.__dict__)
 else:
     # We've been imported for some reason.  Make sure we're setup to work
     # properly, then begin tracing.
